#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import random
from drmaa2 import ReservationSession


def test_get_info():
    rs = ReservationSession('rs-01')
    r_name = 'res-%s' % int(random.uniform(0, 1000))
    r = rs.request_reservation({'reservation_name': r_name, 'duration': 100})
    ri = r.get_info()
    assert (ri.reservation_name == r_name)
    r.terminate()
    print('\nGet info: %s' % (ri))


def test_terminate():
    rs = ReservationSession('rs-01')
    r_name = 'res-%s' % int(random.uniform(0, 1000))
    r = rs.request_reservation({'reservation_name': r_name, 'duration': 100})
    r.terminate()
    print('\nTerminate reservation: %s' % (r))


def test_get_template():
    rs = ReservationSession('rs-01')
    r_name = 'res-%s' % int(random.uniform(0, 1000))
    d = {'reservation_name': r_name, 'duration': 100}
    r = rs.request_reservation(d)
    rt = r.get_template()
    assert (rt.reservation_name == r_name)
    print('\nGet template: %s' % (rt))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from drmaa2 import Version


def test_get_drms_version():
    v = Version.get_drms_version()
    assert (v.major != '' and v.minor != '')
    print('\nGot DRMS version: %s' % (v))


def test_get_drmaa_version():
    v = Version.get_drmaa_version()
    assert (v.major != '' and v.minor != '')
    print('\nGot DRMAA version: %s' % (v))


def test_to_dict():
    v = Version.get_drmaa_version()
    d = v.to_dict()
    assert (v.major == d['major'])
    assert (v.minor == d['minor'])
    print('\nVersion object conversion to dictionary: %s' % (d))


def test_get_implementation_specific_keys():
    keys = Version.get_implementation_specific_keys()
    assert (keys is not None)
    print('\nVersion object has %s impl specific keys %s.' % (len(keys), keys))


def test_get_implementation_specific_attrs():
    attrs = Version.get_implementation_specific_attrs()
    assert (attrs is not None)
    print('\nVersion object has %s impl specific attrs %s.' % (len(attrs), attrs))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import os
import random
from drmaa2 import MonitoringSession
from drmaa2 import JobSession
from drmaa2 import ReservationSession
from drmaa2 import JobInfo
from .utils import needs_uge


@needs_uge
def test_get_all_queues():
    q_name_list = os.popen('qconf -sql').read().split()
    print('\nGot queue list: %s' % q_name_list)
    ms = MonitoringSession('ms-01')
    qi_list = ms.get_all_queues(q_name_list)
    assert (len(qi_list) == len(q_name_list))
    for qi in qi_list:
        q_name = qi.name
        print('Checking queue: %s' % (qi.to_dict()))
        assert (q_name in q_name_list)


def test_get_all_machines():
    h_name_list = os.popen('qconf -sel').read().split()
    print('\nGot host list: %s' % h_name_list)
    ms = MonitoringSession('ms-01')
    mi_list = ms.get_all_machines(h_name_list)
    assert (len(mi_list) == len(h_name_list))
    for mi in mi_list:
        h_name = mi.name
        print('Checking machine: %s' % (mi.to_dict()))
        assert (h_name in h_name_list)


def test_get_all_jobs():
    js = JobSession('js-01')
    j_name = 'drmaa2python-%s' % int(random.uniform(0, 1000))
    j = js.run_job({'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': j_name})
    print('\nSubmitted job: %s' % j)
    ji = j.get_info()
    j.wait_started()
    ms = MonitoringSession('ms-01')
    print('Opened monitoring session: %s' % ms.name)
    ji2 = JobInfo({'job_id': ji.job_id})
    print('Retrieving jobs matching job info %s' % ji2)
    j_list = ms.get_all_jobs(ji2)
    print('Got all jobs: %s' % j_list)
    assert (len(j_list) >= 1)


def test_get_all_reservatios():
    rs = ReservationSession('rs-01')
    r_name = 'drmaa2python-%s' % int(random.uniform(0, 1000))
    d = {'reservation_name': r_name, 'duration': 100}
    r = rs.request_reservation(d)
    print('\nCreated reservation: %s' % r)
    ri = r.get_info()
    # At the moment one cannot have both reservation and monitoring 
    # sessions opened at the same time
    rs.close()

    ms = MonitoringSession('ms-01')
    print('Retrieving reservations matching reservation info %s' % ri)
    r_list = ms.get_all_reservations(ri)
    print('Got all reservations: %s' % r_list)
    assert (len(r_list) == 1)



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from ctypes import POINTER
from ctypes import pointer

from .drmaa2_ctypes import drmaa2_rinfo
from .drmaa2_object import Drmaa2Object
from .drmaa2_exceptions import InvalidArgument


class ReservationInfo(Drmaa2Object):
    """ High-level DRMAA2 reservation info class. """

    reservation_id = Drmaa2Object.StringDescriptor('reservationId')
    """ Reservation id (str). """
    reservation_name = Drmaa2Object.StringDescriptor('reservationName')
    """ Reservation name (str). """
    reserved_start_time = Drmaa2Object.TimeDescriptor('reservedStartTime')
    """ Reserved start time (datetime). """
    reserved_end_time = Drmaa2Object.TimeDescriptor('reservedEndTime')
    """ Reserved end time (datetime). """
    users_acl = Drmaa2Object.StringListDescriptor('usersACL')
    """ Users ACL ([str]). """
    reserved_slots = Drmaa2Object.LongLongDescriptor('reservedSlots')
    """ Reserved slots (long). """
    reserved_machines = Drmaa2Object.StringListDescriptor('reservedMachines')
    """ Reserved machines ([str]). """
    implementation_specific = Drmaa2Object.ImplSpecDescriptor('implementationSpecific')
    """ Implementation specific dictionary ({str:str}). """

    def __init__(self, reservation_info={}):
        """ 
        Constructor. 

        :param reservation_info: Input structure representing the object; this structure is typically a dictionary, but it can also be a low-level drmaa2_rinfo struct.
        :type reservation_info: dict 

        :raises InvalidArgument: in case of an invalid input argument.
        :raises Drmaa2Exception: for all other errors.

        >>> ri = ReservationInfo({'reservation_name' : 'sv-01'})
        >>> print(ri.reservation_name)
        sv-01
        >>> ri.reserved_slots = 3
        """
        Drmaa2Object.__init__(self)
        if isinstance(reservation_info, dict):
            self._struct = self.get_drmaa2_library().drmaa2_rinfo_create()
            self.init_impl_spec_key_values()
            self.from_dict(reservation_info)
        elif isinstance(reservation_info, POINTER(drmaa2_rinfo)):
            self._struct = reservation_info
        else:
            raise InvalidArgument('Invalid argument: %s' % str(reservation_info))

    def __del__(self):
        # self.get_drmaa2_library().drmaa2_rinfo_free(pointer(self._struct))
        pass

    @classmethod
    def get_implementation_specific_keys(cls):
        """
        Retrieve list of implementation-specific keys.

        :returns: String list of implementation-specific keys.

        >>> print(ReservationInfo.get_implementation_specific_keys())
        ['uge_ri_ar_json']
        """
        if cls.implementation_specific_keys is None:
            cls.implementation_specific_keys = cls.to_py_string_list(cls.get_drmaa2_library().drmaa2_rinfo_impl_spec())
        return cls.implementation_specific_keys



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from .utils import generate_random_string
from drmaa2 import JobSession
from drmaa2 import JobState


def test_terminate():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['100']}, 1, 10, 3, 2)
    ja.terminate()
    j_list = ja.job_list
    failed_jobs = []
    for j in j_list:
        s, ss = j.get_state()
        if s == JobState.FAILED:
            failed_jobs.append(j)
    assert (len(failed_jobs) > 0)
    print('\nTerminate job array: %s' % (ja))


def test_suspend_and_resume():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5']}, 1, 10, 3, 2)
    j_list = ja.job_list
    js.wait_any_started(j_list)
    ja.suspend()
    suspended_jobs = []
    for j in j_list:
        s, ss = j.get_state()
        if s == JobState.SUSPENDED:
            suspended_jobs.append(j)
    assert (len(suspended_jobs) > 0)
    print('\nSuspend job array: %s' % (ja))
    ja.resume()
    for j in suspended_jobs:
        s, ss = j.get_state()
        assert (s != JobState.SUSPENDED)
    print('Resume job array: %s' % (ja))


def test_hold_and_release():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5']}, 1, 10, 3, 2)
    ja.hold()
    held_jobs = []
    j_list = ja.job_list
    for j in j_list:
        s, ss = j.get_state()
        if s.name.endswith('HELD'):
            held_jobs.append(j)
    assert (len(held_jobs) > 0)
    print('\nHold job array: %s' % (ja))
    ja.release()
    for j in held_jobs:
        s, ss = j.get_state()
        assert (not s.name.endswith('HELD'))
    print('Release job array: %s' % (ja))


def test_get_template():
    session_name = generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5'], 'job_name': job_name}, 1, 10, 3, 2)
    jt = ja.get_template()
    assert (jt.job_name == job_name)
    print('\nGet template: %s' % (jt))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from nose import SkipTest
from drmaa2 import Notification
from drmaa2 import UnsupportedOperation


def test_register_event_callback():
    def callback(notification):
        print('Got notification: %s' % notification)

    try:
        Notification.register_event_notification(callback)
        print('\nRegistered callback for event notification')
    except UnsupportedOperation:
        raise SkipTest('Not yet implemented optional feature.')



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import random
from .utils import generate_random_string
from .utils import needs_uge
from drmaa2 import ReservationSession


@needs_uge
def test_list_session_names():
    session_names = ReservationSession.list_session_names()
    assert (type(session_names) == type([]))
    print('\nThere are %s existing sessions: %s' % (len(session_names), session_names))


def test_new_session():
    session_name = generate_random_string()
    existing_session_names = ReservationSession.list_session_names()
    rs = ReservationSession(session_name)
    session_names = ReservationSession.list_session_names()
    assert (len(session_names) == len(existing_session_names) + 1)
    print('\nCreated new session: %s' % (session_name))


def test_existing_session():
    session_name = generate_random_string()
    existing_session_names = ReservationSession.list_session_names()
    rs = ReservationSession(session_name, destroy_on_exit=False)
    session_names = ReservationSession.list_session_names()
    assert (len(session_names) == len(existing_session_names) + 1)
    print('\nCreated new session: %s' % (session_name))
    del rs
    rs = ReservationSession(session_name)
    session_names2 = ReservationSession.list_session_names()
    assert (len(session_names) == len(session_names2))
    print('Opened existing session: %s' % (session_name))
    del rs
    session_names3 = ReservationSession.list_session_names()
    assert (len(session_names3) == len(existing_session_names))
    print('Closed session: %s' % (session_name))


def test_destroy_session():
    session_names = ReservationSession.list_session_names()
    print('\nExisting session names: %s' % session_names)
    for name in session_names:
        print('Destroying session: %s' % name)
        ReservationSession.destroy_by_name(name)
    session_names = ReservationSession.list_session_names()
    print('Remaining session names: %s' % session_names)
    assert (len(session_names) == 0)


def test_request_reservation():
    rs = ReservationSession('rs-01')
    r_name = 'reservation-%s' % int(random.uniform(0, 1000))
    r = rs.request_reservation({'reservation_name': r_name, 'duration': 100})
    print('\nRequested reservation: %s' % r)
    ri = r.get_info()
    assert (ri.reservation_name == r_name)
    print('Got reservation info: %s' % ri)


def test_get_reservations():
    rs = ReservationSession('rs-01')
    r_name = 'reservation-%s' % int(random.uniform(0, 1000))
    r = rs.request_reservation({'reservation_name': r_name, 'duration': 100})
    print('\nRequested reservation: %s' % r)
    r_list = rs.get_reservations()
    print('Got reservations: %s' % r_list)
    assert (len(r_list) >= 1)
    r_found = False
    for r2 in r_list:
        if r2.id == r.id:
            r_found = True
            break
    assert (r_found)
    r.terminate()


def test_get_reservation():
    rs = ReservationSession('rs-01')
    r_name = 'reservation-%s' % int(random.uniform(0, 1000))
    r = rs.request_reservation({'reservation_name': r_name, 'duration': 100})
    print(r)
    r2 = rs.get_reservation(r.id)
    assert (r2.id == r.id)
    print('Got reservation: %s' % r)



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import datetime
import socket
from .utils import generate_random_string
from .utils import generate_random_int
from drmaa2 import JobSession
from drmaa2 import JobInfo
from drmaa2 import JobState
from drmaa2.drmaa2_constants import POSIX_EPOCH


def test_job_info_from_job():
    session_name = generate_random_string()
    js = JobSession(session_name)
    jn = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': jn}
    j = js.run_job(d)
    ji = j.get_info()
    assert (ji.job_name == jn)
    print('\nJob info from job: %s' % (ji))


def test_get_implementation_specific_keys():
    keys = JobInfo.get_implementation_specific_keys()
    assert (len(keys) > 0)
    print('\nJob info implementation specific keys: %s' % (keys))


def test_job_id_attr():
    ji = JobInfo()
    job_id = generate_random_string()
    ji.job_id = job_id
    assert (ji.job_id == job_id)
    print('\nJob info with job_id: %s' % (job_id))


def test_job_name_attr():
    ji = JobInfo()
    job_name = generate_random_string()
    ji.job_name = job_name
    assert (ji.job_name == job_name)
    print('\nJob info with job_name: %s' % (job_name))


def test_exit_status_attr():
    ji = JobInfo()
    exit_status = generate_random_int(lower_bound=0, upper_bound=255)
    ji.exit_status = exit_status
    assert (ji.exit_status == exit_status)
    print('\nJob info with exit_status: %s' % (exit_status))


def test_terminating_signal_attr():
    ji = JobInfo()
    terminating_signal = generate_random_string()
    ji.terminating_signal = terminating_signal
    assert (ji.terminating_signal == terminating_signal)
    print('\nJob info with terminating_signal: %s' % (terminating_signal))


def test_annotation_attr():
    ji = JobInfo()
    annotation = generate_random_string()
    ji.annotation = annotation
    assert (ji.annotation == annotation)
    print('\nJob info with annotation: %s' % (annotation))


def test_job_state_attr():
    ji = JobInfo()
    job_state = JobState(generate_random_int(lower_bound=0, upper_bound=9))
    ji.job_state = job_state
    assert (ji.job_state == job_state.name)
    print('\nJob info with job_state: %s' % (job_state))


def test_job_sub_state_attr():
    ji = JobInfo()
    job_sub_state = generate_random_string()
    ji.job_sub_state = job_sub_state
    assert (ji.job_sub_state == job_sub_state)
    print('\nJob info with job_sub_state: %s' % (job_sub_state))


def test_allocated_machines_attr():
    allocated_machines = []
    n_allocated_machines = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_allocated_machines):
        allocated_machines.append(generate_random_string())
    ji = JobInfo()
    ji.allocated_machines = allocated_machines
    assert (ji.allocated_machines == allocated_machines)
    print('\nJob info object with %s allocated_machines: %s' % (len(allocated_machines), allocated_machines))


def test_submission_machine_attr():
    ji = JobInfo()
    submission_machine = socket.gethostname()
    ji.submission_machine = submission_machine
    assert (ji.submission_machine == submission_machine)
    print('\nJob info with submission_machine: %s' % (submission_machine))


def test_job_owner_attr():
    ji = JobInfo()
    job_owner = generate_random_string()
    ji.job_owner = job_owner
    assert (ji.job_owner == job_owner)
    print('\nJob info with job_owner: %s' % (job_owner))


def test_slots_attr():
    ji = JobInfo()
    slots = generate_random_int(lower_bound=0, upper_bound=1024)
    ji.slots = slots
    assert (ji.slots == slots)
    print('\nJob info with slots: %s' % (slots))


def test_queue_name_attr():
    ji = JobInfo()
    queue_name = generate_random_string()
    ji.queue_name = queue_name
    assert (ji.queue_name == queue_name)
    print('\nJob info with queue_name: %s' % (queue_name))


def test_wallclock_time_attr():
    ji = JobInfo()
    wallclock_time = datetime.datetime.fromtimestamp(generate_random_int(lower_bound=0, upper_bound=12386400))
    ji.wallclock_time = wallclock_time
    assert (ji.wallclock_time == wallclock_time)
    print('\nJob info with wallclock_time: %s' % (wallclock_time))


def test_cpu_time_attr():
    ji = JobInfo()
    cpu_time = generate_random_int(lower_bound=0, upper_bound=86400)
    ji.cpu_time = cpu_time
    assert (ji.cpu_time == cpu_time)
    print('\nJob info with cpu_time: %s' % (cpu_time))


def test_submission_time_attr():
    submission_time = datetime.datetime.now()
    ji = JobInfo()
    ji.submission_time = submission_time
    assert (int((ji.submission_time - POSIX_EPOCH).total_seconds()) == int(
        (submission_time - POSIX_EPOCH).total_seconds()))
    print('\nJob info object with submission_time: %s' % (submission_time))


def test_dispatch_time_attr():
    dispatch_time = datetime.datetime.now() + datetime.timedelta(hours=1)
    ji = JobInfo()
    ji.dispatch_time = dispatch_time
    assert (int((ji.dispatch_time - POSIX_EPOCH).total_seconds()) == int((dispatch_time - POSIX_EPOCH).total_seconds()))
    print('\nJob info object with dispatch_time: %s' % (dispatch_time))


def test_finish_time_attr():
    finish_time = datetime.datetime.now() + datetime.timedelta(hours=24)
    ji = JobInfo()
    ji.finish_time = finish_time
    assert (int((ji.finish_time - POSIX_EPOCH).total_seconds()) == int((finish_time - POSIX_EPOCH).total_seconds()))
    print('\nJob info object with finish_time: %s' % (finish_time))


def test_implementation_specific_attr():
    implementation_specific = {}
    keys = JobInfo.get_implementation_specific_keys()
    for k in keys:
        v = generate_random_string()
        implementation_specific[k] = v
    ji = JobInfo()
    ji.implementation_specific = implementation_specific
    assert (ji.implementation_specific == implementation_specific)
    print('\nJob info object with implementation_specific: %s' % (implementation_specific))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from ctypes import cast
from ctypes import POINTER

from .drmaa2_constants import ListType
from .drmaa2_ctypes import drmaa2_string
from .drmaa2_ctypes import drmaa2_r
from .drmaa2_ctypes import drmaa2_list_entryfree

from .sudo import Sudo

from .reservation_template import ReservationTemplate
from .reservation_info import ReservationInfo

from .byte_string import ByteString
from .drmaa2_object import Drmaa2Object
from .log_manager import LogManager
from .exception_mapper import ExceptionMapper


class Reservation(Drmaa2Object):
    """ High-level DRMAA2 reservation class. """

    id = Drmaa2Object.StringDescriptor('id')
    """ Reservation id (str). """
    session_name = Drmaa2Object.StringDescriptor('session_name')
    """ Session name (str). """

    logger = LogManager.get_instance().get_logger('Reservation')

    def __init__(self, reservation):
        """ Constructor. """
        Drmaa2Object.__init__(self)
        if isinstance(reservation, POINTER(drmaa2_r)):
            self._struct = POINTER(drmaa2_r)()
            self._struct.contents = drmaa2_r()
            self.id = ByteString(getattr(reservation.contents, 'id').value).decode()
            self.session_name = ByteString(getattr(reservation.contents, 'session_name').value).decode()
        else:
            raise InvalidArgument('Invalid argument: %s' % str(reservation))

    def terminate(self, auth=None):
        """ 
        Terminate the reservation.

        :param auth: Optional sudo object for terminating the reservation; it can be specified either as a dictionary, or as a Sudo object directly.
        :type auth: Sudo or dict

        >>> r = r_session.request_reservation({'reservation_name' : 'res-01', 'duration' : 100})
        >>> ...
        >>> r.terminate()
        """
        self.logger.debug('Terminating reservation id {}'.format(self.id))
        if auth:
            auth = Sudo.create_from_dict(auth)
            self.logger.debug('Using sudo object: {}'.format(auth))
            ExceptionMapper.check_status_code(self.drmaa2_lib.drmaa2_r_terminate_as(auth._struct, self._struct))
        else:
            ExceptionMapper.check_status_code(self.drmaa2_lib.drmaa2_r_terminate(self._struct))

    def get_template(self):
        """ 
        Get reservation template.

        :returns: ReservationTemplate object.

        >>> r = r_session.request_reservation({'reservation_name' : 'res-01', 'duration' : 100})
        >>> ...
        >>> rt = r.get_template()
        >>> print(rt.users_acl)
        []
        """
        self.logger.debug('Retrieving template for reservation id {}'.format(self.id))
        # ctypes_reservation_template = self.drmaa2_lib.drmaa2_r_get_reservation_template(self._struct)
        ctypes_reservation_template = self.drmaa2_lib.drmaa2_r_get_rtemplate(self._struct)
        if not ctypes_reservation_template:
            ExceptionMapper.check_last_error_code()

        return ReservationTemplate(ctypes_reservation_template)

    def get_info(self):
        """ 
        Get reservation info.

        :returns: ReservationInfo object.

        >>> r = r_session.request_reservation({'reservation_name' : 'res-01', 'duration' : 100})
        >>> ...
        >>> ri = r.get_info()
        >>> print(ri.users_acl)
        []
        """
        self.logger.debug('Retrieving info for reservation id {}'.format(self.id))
        ctypes_reservation_info = self.drmaa2_lib.drmaa2_r_get_info(self._struct)
        if not ctypes_reservation_info:
            ExceptionMapper.check_last_error_code()
        return ReservationInfo(ctypes_reservation_info)

    def __del__(self):
        pass

    @classmethod
    def to_py_reservation_list(cls, ctypes_list):
        py_reservation_list = list()
        if ctypes_list:
            count = cls.drmaa2_lib.drmaa2_list_size(ctypes_list)
            cls.logger.debug('Converting ctypes reservation list of size {}'.format(count))
            for i in range(count):
                void_ptr = cls.drmaa2_lib.drmaa2_list_get(ctypes_list, i)
                if void_ptr:
                    r = cast(void_ptr, POINTER(drmaa2_r))
                    r = Reservation(r)
                    py_reservation_list.append(r)
                else:
                    ExceptionMapper.check_last_error_code()
                    py_reservation_list.append(None)
        return py_reservation_list

    @classmethod
    def to_ctypes_reservation_list(cls, py_reservation_list):
        cls.logger.debug('Converting py reservation list of size {}'.format(len(py_reservation_list)))
        ctypes_reservation_list = cls.drmaa2_lib.drmaa2_list_create(int(ListType.RESERVATIONLIST),
                                                                    drmaa2_list_entryfree())
        for r in py_reservation_list:
            ExceptionMapper.check_status_code(cls.drmaa2_lib.drmaa2_list_add(ctypes_reservation_list, j._struct))
        return ctypes_reservation_list



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from .utils import generate_random_string
from drmaa2 import JobSession
from drmaa2 import JobState


def test_get_info():
    session_name = 'drmaa2python-%s' % generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': job_name}
    j = js.run_job(d)
    j.wait_started()
    ji = j.get_info()
    assert (ji.job_name == job_name)
    print('\nGet info: %s' % (ji))


def test_get_state():
    session_name = 'drmaa2python-%s' % generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': job_name}
    j = js.run_job(d)
    (j_state, j_sub_state) = j.get_state()
    assert (isinstance(j_state, JobState))
    print('\nGet state %s for job %s' % (j_state, j))


def test_terminate():
    session_name = 'drmaa2python-%s' % generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['100'], 'job_name': job_name}
    j = js.run_job(d)
    ji = j.get_info()
    assert (ji.terminating_signal is None)
    j.wait_started()
    j.terminate()
    j.wait_terminated()
    ji = j.get_info()
    assert (ji.terminating_signal is not None)
    print('\nTerminate job: %s' % (ji))


def test_suspend_and_resume():
    session_name = 'drmaa2python-%s' % generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': job_name}
    j = js.run_job(d)
    j.wait_started()
    ji = j.get_info()
    assert (ji.job_state != JobState.SUSPENDED.name)
    j.suspend()
    ji = j.get_info()
    assert (ji.job_state == JobState.SUSPENDED.name)
    print('\nSuspend job: %s' % (ji))
    j.resume()
    ji = j.get_info()
    assert (ji.job_state != JobState.SUSPENDED.name)
    print('Resume job: %s' % (ji))


def test_hold_and_release():
    session_name = 'drmaa2python-%s' % generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': job_name}
    j = js.run_job(d)
    ji = j.get_info()
    assert (not ji.job_state.endswith('HELD'))
    j.hold()
    ji = j.get_info()
    assert (ji.job_state.endswith('HELD'))
    print('\nHold job: %s' % (ji))
    j.release()
    ji = j.get_info()
    assert (not ji.job_state.endswith('HELD'))
    print('Release job: %s' % (ji))


def test_wait_started():
    session_name = 'drmaa2python-%s' % generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': job_name}
    j = js.run_job(d)
    j.wait_started()
    s, ss = j.get_state()
    assert (s == JobState.RUNNING)
    print('\nWait started for job: %s' % (j))


def test_wait_terminated():
    session_name = 'drmaa2python-%s' % generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': job_name}
    j = js.run_job(d)
    j.wait_terminated()
    s, ss = j.get_state()
    assert (s == JobState.DONE)
    print('\nWait terminated for job: %s' % (j))


def test_get_template():
    session_name = 'drmaa2python-%s' % generate_random_string()
    js = JobSession(session_name)
    job_name = 'drmaa2python-%s' % generate_random_string()
    d = {'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': job_name}
    j = js.run_job(d)
    jt = j.get_template()
    assert (jt.job_name == job_name)
    print('\nGet template: %s' % (jt))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from drmaa2 import LibraryManager
from drmaa2 import Capability


def test_get_drms_name():
    lm = LibraryManager.get_instance()
    n = lm.get_drms_name()
    assert (n != '')
    print('\nGot DRMS name: %s' % (n))


def test_get_drmaa_name():
    lm = LibraryManager.get_instance()
    n = lm.get_drmaa_name()
    assert (n != '')
    print('\nGot DRMAA name: %s' % (n))


def test_drmaa_supports():
    lm = LibraryManager.get_instance()
    print('\nVerifying support for %s capabilities' % (len(Capability)))
    supported = []
    for c in Capability:
        s = lm.drmaa_supports(c)
        if s:
            supported.append(s)
        print('Support for %s: %s' % (c, s))
    assert (len(supported) > 0)
    print('DRMAA2 supports %s out of %s capabilities' % (len(supported), len(Capability)))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import os
from drmaa2 import MonitoringSession


def test_machine_info():
    host_list = os.popen('qconf -sel').read().split()
    print('\nGot hosts: %s' % host_list)
    h = host_list[0]
    print('\nUsing first host: %s' % h)
    ms = MonitoringSession('ms-01')
    mi_list = ms.get_all_machines([h])
    print(mi_list)
    assert (len(mi_list) == 1)
    mi = mi_list[0]
    assert (mi.name == h)
    print('\nGot machine info: %s' % (mi.to_dict()))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from ctypes import pointer
from ctypes import POINTER

from .drmaa2_constants import Bool
from .drmaa2_constants import Time
from .drmaa2_constants import JobState
from .drmaa2_ctypes import drmaa2_string
from .drmaa2_ctypes import drmaa2_jarray

from .sudo import Sudo

from .byte_string import ByteString
from .job import Job
from .job_info import JobInfo
from .job_template import JobTemplate
from .drmaa2_object import Drmaa2Object
from .log_manager import LogManager
from .exception_mapper import ExceptionMapper
from .drmaa2_job_list_descriptor import Drmaa2JobListDescriptor


class JobArray(Drmaa2Object):
    """ High-level DRMAA2 job array class. """

    id = Drmaa2Object.StringDescriptor('id')
    """ Job id (str). """
    session_name = Drmaa2Object.StringDescriptor('session_name')
    """ Session name (str). """
    job_list = Drmaa2JobListDescriptor('job_list')
    """ Job list ([Job]). """

    logger = LogManager.get_instance().get_logger('Job')

    def __init__(self, struct=None):
        """ Constructor. """
        Drmaa2Object.__init__(self, struct)

    def suspend(self, auth=None):
        """ 
        Suspend the job array.

        :param auth: Optional sudo object for suspending the job array; it can be specified either as a dictionary, or as a Sudo object directly.
        :type auth: Sudo or dict

        >>> ja = j_session.run_bulk_jobs({'remote_command' : '/bin/sleep', 'args' : ['10']}, 1, 10, 3, 2)
        >>> ...
        >>> ja.suspend()
        """
        self.logger.debug('Suspending job array id {}'.format(self.id))
        drmaa2_lib = self.get_drmaa2_library()
        if auth:
            auth = Sudo.create_from_dict(auth)
            self.logger.debug('Using sudo object: {}'.format(auth))
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_suspend_as(auth._struct, self._struct))
        else:
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_suspend(self._struct))

    def resume(self, auth=None):
        """ 
        Resume the job array.

        :param auth: Optional sudo object for resuming the job array; it can be specified either as a dictionary, or as a Sudo object directly.
        :type auth: Sudo or dict

        >>> ja = j_session.run_bulk_jobs({'remote_command' : '/bin/sleep', 'args' : ['10']}, 1, 10, 3, 2)
        >>> ...
        >>> ja.suspend()
        >>> ...
        >>> ja.resume()
        """
        self.logger.debug('Resuming job array id {}'.format(self.id))
        drmaa2_lib = self.get_drmaa2_library()
        if auth:
            auth = Sudo.create_from_dict(auth)
            self.logger.debug('Using sudo object: {}'.format(auth))
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_resume_as(auth._struct, self._struct))
        else:
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_resume(self._struct))

    def hold(self, auth=None):
        """ 
        Hold the job array.

        :param auth: Optional sudo object for holding the job array; it can be specified either as a dictionary, or as a Sudo object directly.
        :type auth: Sudo or dict

        >>> ja = j_session.run_bulk_jobs({'remote_command' : '/bin/sleep', 'args' : ['10']}, 1, 10, 3, 2)
        >>> ...
        >>> ja.hold()
        """
        self.logger.debug('Holding job array id {}'.format(self.id))
        drmaa2_lib = self.get_drmaa2_library()
        if auth:
            auth = Sudo.create_from_dict(auth)
            self.logger.debug('Using sudo object: {}'.format(auth))
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_hold_as(auth._struct, self._struct))
        else:
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_hold(self._struct))

    def release(self, auth=None):
        """ 
        Release the job array.

        :param auth: Optional sudo object for releasing the job array; it can be specified either as a dictionary, or as a Sudo object directly.
        :type auth: Sudo or dict

        >>> ja = j_session.run_bulk_jobs({'remote_command' : '/bin/sleep', 'args' : ['10']}, 1, 10, 3, 2)
        >>> ...
        >>> ja.hold()
        >>> ...
        >>> ja.release()
        """
        self.logger.debug('Releasing job array id {}'.format(self.id))
        drmaa2_lib = self.get_drmaa2_library()
        if auth:
            auth = Sudo.create_from_dict(auth)
            self.logger.debug('Using sudo object: {}'.format(auth))
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_release_as(auth._struct, self._struct))
        else:
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_release(self._struct))

    def terminate(self, auth=None):
        """ 
        Terminate the job array.

        :param auth: Optional sudo object for terminating the job array; it can be specified either as a dictionary, or as a Sudo object directly.
        :type auth: Sudo or dict

        >>> ja = j_session.run_bulk_jobs({'remote_command' : '/bin/sleep', 'args' : ['100']}, 1, 10, 3, 1)
        >>> ...
        >>> ja.terminate()
        """
        self.logger.debug('Terminating job array id {}'.format(self.id))
        drmaa2_lib = self.get_drmaa2_library()
        if auth:
            auth = Sudo.create_from_dict(auth)
            self.logger.debug('Using sudo object: {}'.format(auth))
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_terminate_as(auth._struct, self._struct))
        else:
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_terminate(self._struct))

    def terminate_all(self, auth=None):
        """ 
        Terminate all job array tasks.

        :param auth: Optional sudo object for terminating job array tasks; it can be specified either as a dictionary, or as a Sudo object directly.
        :type auth: Sudo or dict

        >>> ja = j_session.run_bulk_jobs({'remote_command' : '/bin/sleep', 'args' : ['100']}, 1, 10, 3, 3)
        >>> ...
        >>> ja.terminate_all()
        """
        self.logger.debug('Terminating all tasks for job array id {}'.format(self.id))
        drmaa2_lib = self.get_drmaa2_library()
        if auth:
            auth = Sudo.create_from_dict(auth)
            self.logger.debug('Using sudo object: {}'.format(auth))
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_terminate_all_as(auth._struct, self._struct))
        else:
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_terminate_all(self._struct))

    def reap(self):
        """ 
        Reap the job array from internal lists.

        >>> ja = j_session.run_bulk_jobs({'remote_command' : '/bin/sleep', 'args' : ['10']}, 1, 10, 3, 3)
        >>> ...
        >>> ja.reap()
        """
        self.logger.debug('Reaping job array id {}'.format(self.id))
        drmaa2_lib = self.get_drmaa2_library()
        ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_jarray_reap(self._struct))

    def get_template(self):
        """ 
        Get job array template.

        :returns: JobTemplate object.

        >>> ja = j_session.run_bulk_jobs({'remote_command' : '/bin/sleep', 'args' : ['100']}, 1, 10, 5, 2)
        >>> ...
        >>> jt = ja.get_template()
        >>> print(jt.remote_command)
        /bin/sleep
        >>> print(jt.args)
        ['100']
        """
        self.logger.debug('Retrieving template for job array id {}'.format(self.id))
        drmaa2_lib = self.get_drmaa2_library()
        ctypes_job_template = drmaa2_lib.drmaa2_jarray_get_jtemplate(self._struct)
        if not ctypes_job_template:
            self.exception_mapper.check_last_error_code()

        return JobTemplate(ctypes_job_template)

    def __del__(self):
        if self._struct:
            self.get_drmaa2_library().drmaa2_jarray_free(pointer(self._struct))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import datetime
from .utils import generate_random_string
from .utils import generate_random_int
from drmaa2 import JobTemplate
from drmaa2 import Bool
from drmaa2 import Os
from drmaa2 import Cpu
from drmaa2 import Drmaa2Exception
from drmaa2.drmaa2_constants import POSIX_EPOCH


def test_remote_command_attr():
    remote_command = generate_random_string()
    jt = JobTemplate()
    jt.remote_command = remote_command
    assert (jt.remote_command == remote_command)
    print('\nJob template object with remote_command: %s' % (remote_command))


def test_args_attr():
    args = []
    n_args = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_args):
        args.append(generate_random_string())
    jt = JobTemplate()
    jt.args = args
    assert (jt.args == args)
    print('\nJob template object with %s args: %s' % (len(args), args))


def test_submit_as_hold_attr():
    submit_as_hold = generate_random_int(lower_bound=0, upper_bound=1)
    jt = JobTemplate()
    jt.submit_as_hold = submit_as_hold
    assert (jt.submit_as_hold == Bool(submit_as_hold).name)
    print('\nJob template object with submit_as_hold: %s' % (submit_as_hold))


def test_rerunnable_attr():
    rerunnable = generate_random_int(lower_bound=0, upper_bound=1)
    jt = JobTemplate()
    jt.rerunnable = rerunnable
    assert (jt.rerunnable == Bool(rerunnable).name)
    print('\nJob template object with rerunnable: %s' % (rerunnable))


def test_job_environment_attr():
    job_environment = {}
    n_args = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_args):
        name = generate_random_string().upper()
        value = generate_random_string()
        job_environment[name] = value
    jt = JobTemplate()
    jt.job_environment = job_environment
    assert (jt.job_environment == job_environment)
    print('\nJob template object with job_environment: %s' % (job_environment))


def test_working_directory_attr():
    working_directory = generate_random_string()
    jt = JobTemplate()
    jt.working_directory = working_directory
    assert (jt.working_directory == working_directory)
    print('\nJob template object with working_directory: %s' % (working_directory))


def test_job_category_attr():
    job_category = generate_random_string()
    jt = JobTemplate()
    jt.job_category = job_category
    assert (jt.job_category == job_category)
    print('\nJob template object with job_category: %s' % (job_category))


def test_email_attr():
    email = []
    n_email = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_email):
        email.append(generate_random_string())
    jt = JobTemplate()
    jt.email = email
    assert (jt.email == email)
    print('\nJob template object with %s emails: %s' % (len(email), email))


def test_email_on_started_attr():
    email_on_started = generate_random_int(lower_bound=0, upper_bound=1)
    jt = JobTemplate()
    jt.email_on_started = email_on_started
    assert (jt.email_on_started == Bool(email_on_started).name)
    print('\nJob template object with email_on_started: %s' % (email_on_started))


def test_email_on_terminated_attr():
    email_on_terminated = generate_random_int(lower_bound=0, upper_bound=1)
    jt = JobTemplate()
    jt.email_on_terminated = email_on_terminated
    assert (jt.email_on_terminated == Bool(email_on_terminated).name)
    print('\nJob template object with email_on_terminated: %s' % (email_on_terminated))


def test_job_name_attr():
    job_name = generate_random_string()
    jt = JobTemplate()
    jt.job_name = job_name
    assert (jt.job_name == job_name)
    print('\nJob template object with job_name: %s' % (job_name))


def test_input_path_attr():
    input_path = generate_random_string()
    jt = JobTemplate()
    jt.input_path = input_path
    assert (jt.input_path == input_path)
    print('\nJob template object with input_path: %s' % (input_path))


def test_output_path_attr():
    output_path = generate_random_string()
    jt = JobTemplate()
    jt.output_path = output_path
    assert (jt.output_path == output_path)
    print('\nJob template object with output_path: %s' % (output_path))


def test_error_path_attr():
    error_path = generate_random_string()
    jt = JobTemplate()
    jt.error_path = error_path
    assert (jt.error_path == error_path)
    print('\nJob template object with error_path: %s' % (error_path))


def test_join_files_attr():
    join_files = generate_random_int(lower_bound=0, upper_bound=1)
    jt = JobTemplate()
    jt.join_files = join_files
    assert (jt.join_files == Bool(join_files).name)
    print('\nJob template object with join_files: %s' % (join_files))


def test_reservation_id_attr():
    reservation_id = generate_random_string()
    jt = JobTemplate()
    jt.reservation_id = reservation_id
    assert (jt.reservation_id == reservation_id)
    print('\nJob template object with reservation_id: %s' % (reservation_id))


def test_queue_name_attr():
    queue_name = generate_random_string()
    jt = JobTemplate()
    jt.queue_name = queue_name
    assert (jt.queue_name == queue_name)
    print('\nJob template object with queue_name: %s' % (queue_name))


def test_min_slots_attr():
    min_slots = generate_random_int(lower_bound=0, upper_bound=100)
    jt = JobTemplate()
    jt.min_slots = min_slots
    assert (jt.min_slots == min_slots)
    print('\nJob template object with min_slots: %s' % (min_slots))


def test_max_slots_attr():
    max_slots = generate_random_int(lower_bound=0, upper_bound=100)
    jt = JobTemplate()
    jt.max_slots = max_slots
    assert (jt.max_slots == max_slots)
    print('\nJob template object with max_slots: %s' % (max_slots))


def test_priority_attr():
    priority = generate_random_int(lower_bound=0, upper_bound=1000)
    jt = JobTemplate()
    jt.priority = priority
    assert (jt.priority == priority)
    print('\nJob template object with priority: %s' % (priority))


def test_candidate_machines_attr():
    candidate_machines = []
    n_candidate_machines = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_candidate_machines):
        candidate_machines.append(generate_random_string())
    jt = JobTemplate()
    jt.candidate_machines = candidate_machines
    assert (jt.candidate_machines == candidate_machines)
    print('\nJob template object with %s candidate_machines: %s' % (len(candidate_machines), candidate_machines))


def test_min_phys_memory_attr():
    min_phys_memory = generate_random_int(lower_bound=0, upper_bound=1000)
    jt = JobTemplate()
    jt.min_phys_memory = min_phys_memory
    assert (jt.min_phys_memory == min_phys_memory)
    print('\nJob template object with min_phys_memory: %s' % (min_phys_memory))


def test_machine_os_attr():
    machine_os = generate_random_int(lower_bound=0, upper_bound=11)
    jt = JobTemplate()
    jt.machine_os = machine_os
    assert (jt.machine_os == Os(machine_os).name)
    print('\nJob template object with machine_os: %s' % (machine_os))


def test_machine_arch_attr():
    machine_arch = generate_random_int(lower_bound=0, upper_bound=16)
    jt = JobTemplate()
    jt.machine_arch = machine_arch
    assert (jt.machine_arch == Cpu(machine_arch).name)
    print('\nJob template object with machine_arch: %s' % (machine_arch))


def test_start_time_attr():
    start_time = datetime.datetime.now()
    jt = JobTemplate()
    jt.start_time = start_time
    assert (int((jt.start_time - POSIX_EPOCH).total_seconds()) == int((start_time - POSIX_EPOCH).total_seconds()))
    print('\nJob template object with start_time: %s' % (start_time))


def test_deadline_time_attr():
    deadline_time = datetime.datetime.now() + datetime.timedelta(hours=1)
    jt = JobTemplate()
    jt.deadline_time = deadline_time
    assert (int((jt.deadline_time - POSIX_EPOCH).total_seconds()) == int((deadline_time - POSIX_EPOCH).total_seconds()))
    print('\nJob template object with deadline_time: %s' % (deadline_time))


def test_stage_in_files_attr():
    stage_in_files = {}
    n_args = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_args):
        name = generate_random_string().upper()
        value = generate_random_string()
        stage_in_files[name] = value
    jt = JobTemplate()
    jt.stage_in_files = stage_in_files
    assert (jt.stage_in_files == stage_in_files)
    print('\nJob template object with stage_in_files: %s' % (stage_in_files))


def test_stage_out_files_attr():
    stage_out_files = {}
    n_args = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_args):
        name = generate_random_string().upper()
        value = generate_random_string()
        stage_out_files[name] = value
    jt = JobTemplate()
    jt.stage_out_files = stage_out_files
    assert (jt.stage_out_files == stage_out_files)
    print('\nJob template object with stage_out_files: %s' % (stage_out_files))


def test_resource_limits_attr():
    resource_limits = {}
    n_args = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_args):
        name = generate_random_string().upper()
        value = generate_random_string()
        resource_limits[name] = value
    jt = JobTemplate()
    jt.resource_limits = resource_limits
    assert (jt.resource_limits == resource_limits)
    print('\nJob template object with resource_limits: %s' % (resource_limits))


def test_accounting_id_attr():
    accounting_id = generate_random_string()
    jt = JobTemplate()
    jt.accounting_id = accounting_id
    assert (jt.accounting_id == accounting_id)
    print('\nJob template object with accounting_id: %s' % (accounting_id))


def test_implementation_specific_attr():
    implementation_specific = {}
    keys = JobTemplate.get_implementation_specific_keys()
    for k in keys:
        v = generate_random_string()
        implementation_specific[k] = v
    jt = JobTemplate()
    jt.implementation_specific = implementation_specific
    assert (jt.implementation_specific == implementation_specific)
    print('\nJob template object with implementation_specific: %s' % (implementation_specific))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import datetime
from .utils import generate_random_string
from .utils import generate_random_int
from drmaa2 import ReservationSession
from drmaa2 import ReservationTemplate
from drmaa2 import Os
from drmaa2 import Cpu
from drmaa2.drmaa2_constants import POSIX_EPOCH


def test_reservation_template_from_reservation():
    rs = ReservationSession('rs-01')
    reservation_name = 'r.%s' % generate_random_string()
    r = rs.request_reservation({'reservation_name': reservation_name, 'duration': 100})
    rt = r.get_template()
    assert (rt.reservation_name == reservation_name)
    print('\nReservation template from reservation: %s' % (rt))


def test_get_implementation_specific_keys():
    keys = ReservationTemplate.get_implementation_specific_keys()
    assert (len(keys) > 0)
    print('\nReservation template implementation specific keys: %s' % (keys))


def test_reservation_name_attr():
    rt = ReservationTemplate()
    reservation_name = generate_random_string()
    rt.reservation_name = reservation_name
    assert (rt.reservation_name == reservation_name)
    print('\nReservation template with reservation_name: %s' % (reservation_name))


def test_start_time_attr():
    start_time = datetime.datetime.now()
    rt = ReservationTemplate()
    rt.start_time = start_time
    assert (int((rt.start_time - POSIX_EPOCH).total_seconds()) == int((start_time - POSIX_EPOCH).total_seconds()))
    print('\nReservation template object with start_time: %s' % (start_time))


def test_end_time_attr():
    end_time = datetime.datetime.now() + datetime.timedelta(hours=24)
    rt = ReservationTemplate()
    rt.end_time = end_time
    assert (int((rt.end_time - POSIX_EPOCH).total_seconds()) == int((end_time - POSIX_EPOCH).total_seconds()))
    print('\nReservation template object with end_time: %s' % (end_time))


def test_duration_attr():
    rt = ReservationTemplate()
    duration = generate_random_int(lower_bound=1, upper_bound=86400)
    rt.duration = duration
    assert (rt.duration == duration)
    print('\nReservation template with duration: %s' % (duration))


def test_min_slots_attr():
    rt = ReservationTemplate()
    min_slots = generate_random_int(lower_bound=1, upper_bound=1024)
    rt.min_slots = min_slots
    assert (rt.min_slots == min_slots)
    print('\nReservation template with min_slots: %s' % (min_slots))


def test_max_slots_attr():
    rt = ReservationTemplate()
    max_slots = generate_random_int(lower_bound=1, upper_bound=1024)
    rt.max_slots = max_slots
    assert (rt.max_slots == max_slots)
    print('\nReservation template with max_slots: %s' % (max_slots))


def test_job_category_attr():
    rt = ReservationTemplate()
    job_category = generate_random_string()
    rt.job_category = job_category
    assert (rt.job_category == job_category)
    print('\nReservation template with job_category: %s' % (job_category))


def test_users_acl_attr():
    users_acl = []
    n_users_acl = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_users_acl):
        users_acl.append(generate_random_string())
    rt = ReservationTemplate()
    rt.users_acl = users_acl
    assert (rt.users_acl == users_acl)
    print('\nreservation template object with %s users_acl: %s' % (len(users_acl), users_acl))


def test_candidate_machines_attr():
    candidate_machines = []
    n_candidate_machines = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_candidate_machines):
        candidate_machines.append(generate_random_string())
    rt = ReservationTemplate()
    rt.candidate_machines = candidate_machines
    assert (rt.candidate_machines == candidate_machines)
    print(
        '\nreservation template object with %s candidate_machines: %s' % (len(candidate_machines), candidate_machines))


def test_min_phys_memory_attr():
    rt = ReservationTemplate()
    min_phys_memory = generate_random_int(lower_bound=1, upper_bound=86400)
    rt.min_phys_memory = min_phys_memory
    assert (rt.min_phys_memory == min_phys_memory)
    print('\nReservation template with min_phys_memory: %s' % (min_phys_memory))


def test_machine_os_attr():
    rt = ReservationTemplate()
    machine_os = generate_random_int(lower_bound=0, upper_bound=11)
    rt.machine_os = machine_os
    assert (rt.machine_os == Os(machine_os).name)
    print('\nReservation template with machine_os: %s' % (machine_os))


def test_machine_arch_attr():
    machine_arch = generate_random_int(lower_bound=0, upper_bound=16)
    rt = ReservationTemplate()
    rt.machine_arch = machine_arch
    assert (rt.machine_arch == Cpu(machine_arch).name)
    print('\nReservation template object with machine_arch: %s' % (machine_arch))


def test_implementation_specific_attr():
    implementation_specific = {}
    keys = ReservationTemplate.get_implementation_specific_keys()
    for k in keys:
        v = generate_random_string()
        implementation_specific[k] = v
    rt = ReservationTemplate()
    rt.implementation_specific = implementation_specific
    assert (rt.implementation_specific == implementation_specific)
    print('\nReservation template object with implementation_specific: %s' % (implementation_specific))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import os
from drmaa2 import MonitoringSession


def test_queue_info():
    q_name_list = os.popen('qconf -sql').read().split()
    print('\nGot queue list: %s' % q_name_list)
    ms = MonitoringSession('ms-01')
    qi_list = ms.get_all_queues(q_name_list)
    assert (len(qi_list) > 0)
    for qi in qi_list:
        q_name = qi.name
        print('\nChecking queue: %s' % (qi.to_dict()))
        assert (q_name in q_name_list)



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import types
import datetime
from ctypes import cast

from .byte_string import ByteString

from .drmaa2_constants import UNSET_BOOL
from .drmaa2_constants import UNSET_NUM
from .drmaa2_constants import UNSET_ENUM
from .drmaa2_constants import POSIX_EPOCH
from .drmaa2_constants import PY_STRING_TYPE

from .drmaa2_constants import ListType
from .drmaa2_constants import Bool
from .drmaa2_constants import Time

from .drmaa2_exceptions import UnsupportedAttribute

from .drmaa2_ctypes import drmaa2_string
from .drmaa2_ctypes import drmaa2_list
from .drmaa2_ctypes import drmaa2_string_list
from .drmaa2_ctypes import drmaa2_list_entryfree
from .drmaa2_ctypes import drmaa2_dict_entryfree

from .log_manager import LogManager
from .library_manager import LibraryManager
from .exception_mapper import ExceptionMapper


class Drmaa2Descriptor(object):
    """ Base descriptor class for drmaa2_object fields. """

    drmaa2_lib = None

    def __init__(self, name):
        self.name = name

    @classmethod
    def get_drmaa2_library(cls):
        if not cls.drmaa2_lib:
            cls.drmaa2_lib = LibraryManager.get_instance().get_drmaa2_library()
        return cls.drmaa2_lib

    @classmethod
    def get_contents(cls, obj):
        if obj._struct is not None:
            if hasattr(obj._struct, 'contents'):
                return obj._struct.contents
            else:
                return obj._struct
        else:
            return None

    @classmethod
    def can_read(cls, obj):
        if obj is not None and obj._struct is not None:
            return True
        return False

    @classmethod
    def can_write(cls, obj):
        if obj is not None and obj._struct is not None and not obj._read_only:
            return True
        return False


class Drmaa2BoolDescriptor(Drmaa2Descriptor):
    """ A descriptor for drmaa2_bool fields. """

    def __init__(self, name):
        Drmaa2Descriptor.__init__(self, name)

    def __get__(self, obj, type=None):
        value = False
        if self.can_read(obj):
            value = getattr(obj._struct.contents, self.name)
        return Bool(value).name

    def __set__(self, obj, value):
        if not self.can_write(obj):
            return

        if value is not None:
            if type(value) == PY_STRING_TYPE:
                value = Bool[value].value
            elif value:
                value = int(Bool.TRUE)
            else:
                value = int(Bool.FALSE)
        else:
            value = UNSET_BOOL
        setattr(obj._struct.contents, self.name, value)
        obj._dict[self.name] = value


class Drmaa2EnumDescriptor(Drmaa2Descriptor):
    """ A descriptor for drmaa2_enum fields. """

    def __init__(self, name, cls):
        Drmaa2Descriptor.__init__(self, name)
        self.cls = cls

    def __get__(self, obj, type=None):
        value = UNSET_ENUM
        if self.can_read(obj):
            value = getattr(obj._struct.contents, self.name)
        if value == UNSET_ENUM:
            return None
        return self.cls(value).name

    def __set__(self, obj, value):
        if not self.can_write(obj):
            return
        if value is not None:
            if type(value) == PY_STRING_TYPE:
                value = self.cls[value].value
        else:
            value = UNSET_ENUM
        setattr(obj._struct.contents, self.name, value)
        obj._dict[self.name] = value


class Drmaa2TimeDescriptor(Drmaa2Descriptor):
    """ A descriptor for drmaa2_enum fields. """

    def __init__(self, name):
        Drmaa2Descriptor.__init__(self, name)

    def __get__(self, obj, type=None):
        value = int(Time.UNSET_TIME)
        if self.can_read(obj):
            value = getattr(obj._struct.contents, self.name)
        try:
            t = Time(value)
            if t == Time.UNSET_TIME:
                return None
            else:
                return t.name
        except ValueError:
            return datetime.datetime.utcfromtimestamp(value)
            # return datetime.datetime.fromtimestamp(value)

    def __set__(self, obj, value):
        if not self.can_write(obj):
            return
        if value is not None:
            if type(value) == PY_STRING_TYPE:
                value = int(Time[value])
            elif isinstance(value, datetime.datetime):
                value = int((value - POSIX_EPOCH).total_seconds())
            else:
                value = int(value)
        else:
            value = int(Time.UNSET_TIME)
        setattr(obj._struct.contents, self.name, value)
        obj._dict[self.name] = value


class Drmaa2NumericTypeDescriptor(Drmaa2Descriptor):
    """ A descriptor for numeric type fields. """

    def __init__(self, name, unset_value=UNSET_NUM):
        Drmaa2Descriptor.__init__(self, name)
        self.unset_value = unset_value

    def __get__(self, obj, type=None):
        value = self.unset_value
        if self.can_read(obj):
            value = getattr(obj._struct.contents, self.name)
        if value == self.unset_value:
            value = None
        return value

    def __set__(self, obj, value):
        if not self.can_write(obj):
            return
        value = value if value is not None else self.unset_value
        setattr(obj._struct.contents, self.name, value)
        obj._dict[self.name] = value


class Drmaa2IntDescriptor(Drmaa2NumericTypeDescriptor):
    """ A descriptor for int fields. """

    def __init__(self, name, unset_value=UNSET_NUM):
        Drmaa2NumericTypeDescriptor.__init__(self, name, unset_value)


class Drmaa2LongDescriptor(Drmaa2IntDescriptor):
    """ A descriptor for long fields. """

    def __init__(self, name, unset_value=UNSET_NUM):
        Drmaa2IntDescriptor.__init__(self, name, unset_value)


class Drmaa2LongLongDescriptor(Drmaa2IntDescriptor):
    """ A descriptor for long long fields. """

    def __init__(self, name, unset_value=UNSET_NUM):
        Drmaa2IntDescriptor.__init__(self, name, unset_value)


class Drmaa2FloatDescriptor(Drmaa2NumericTypeDescriptor):
    """ A descriptor for float fields. """

    def __init__(self, name, unset_value=UNSET_NUM):
        Drmaa2NumericTypeDescriptor.__init__(self, name, unset_value)


class Drmaa2CharBufferDescriptor(Drmaa2Descriptor):
    """ A descriptor for char[] fields. """

    def __init__(self, name):
        Drmaa2Descriptor.__init__(self, name)

    def __get__(self, obj, type=None):
        value = None
        if self.can_read(obj):
            value = getattr(obj._struct.contents, self.name)
            if hasattr(value, 'value'):
                value = value.value
        return ByteString(value).decode()

    def __set__(self, obj, value):
        if not self.can_write(obj):
            return
        if value is None:
            return
        value = ByteString(value).encode()
        setattr(obj._struct.contents, self.name, value)
        obj._dict[self.name] = value


class Drmaa2StringDescriptor(Drmaa2Descriptor):
    """ A descriptor for drmaa2_string fields. """

    def __init__(self, name):
        Drmaa2Descriptor.__init__(self, name)

    def __get__(self, obj, type=None):
        value = None
        if self.can_read(obj):
            value = getattr(obj._struct.contents, self.name)
            if hasattr(value, 'value'):
                value = value.value
        return ByteString(value).decode()

    def __set__(self, obj, value):
        if not self.can_write(obj):
            return
        if value is not None:
            value = ByteString(value).encode()
        else:
            value = ByteString(drmaa2_string()).encode()
        setattr(obj._struct.contents, self.name, value)
        obj._dict[self.name] = value


class Drmaa2StringListDescriptor(Drmaa2Descriptor):
    """ A descriptor for drmaa2_string_list fields. """

    logger = LogManager.get_instance().get_logger('Drmaa2StringListDescriptor')

    def __init__(self, name):
        Drmaa2Descriptor.__init__(self, name)

    def __get__(self, obj, type=None):
        if not self.can_read(obj):
            return None
        value_list = list()
        ctypes_list = getattr(obj._struct.contents, self.name)
        if ctypes_list:
            count = self.get_drmaa2_library().drmaa2_list_size(ctypes_list)
            self.logger.debug('Converting ctypes list {} of size {}'.format(self.name, count))
            for i in range(count):
                void_ptr = self.get_drmaa2_library().drmaa2_list_get(ctypes_list, i)
                if void_ptr:
                    value = ByteString(cast(void_ptr, drmaa2_string).value).decode()
                    self.logger.debug('{}[{}] = {}'.format(self.name, i, value))
                    value_list.append(value)
                else:
                    ExceptionMapper.check_last_error_code()
                    value_list.append(None)
        return value_list

    def __set__(self, obj, value_list):
        if not self.can_write(obj):
            return
        if value_list is None:
            value_list = []
        ctypes_list = getattr(obj._struct.contents, self.name)
        if ctypes_list:
            count = self.get_drmaa2_library().drmaa2_list_size(ctypes_list)
            self.logger.debug('Clearing string list {} (size {})'.format(self.name, count))
            while count > 0:
                ExceptionMapper.check_status_code(self.get_drmaa2_library().drmaa2_list_del(ctypes_list, 0))
                count = self.get_drmaa2_library().drmaa2_list_size(ctypes_list)
        else:
            self.logger.debug('Creating string list {}'.format(self.name))
            ctypes_list = self.get_drmaa2_library().drmaa2_list_create(int(ListType.STRINGLIST),
                                                                       drmaa2_list_entryfree())
            setattr(obj._struct.contents, self.name, ctypes_list)

        value_list = [ByteString(v).encode() for v in value_list]
        for i in range(len(value_list)):
            v = value_list[i]
            self.logger.debug('Adding {}[{}] = {}'.format(self.name, i, v))
            ExceptionMapper.check_status_code(self.get_drmaa2_library().drmaa2_list_add(ctypes_list, v))
        # this assures proper memory management in python 3
        obj._dict[self.name] = value_list


class Drmaa2DictDescriptor(Drmaa2Descriptor):
    """ A descriptor for drmaa2_dict fields. """

    logger = LogManager.get_instance().get_logger('Drmaa2DictDescriptor')

    def __init__(self, name):
        Drmaa2Descriptor.__init__(self, name)

    def __get__(self, obj, type=None):
        if not self.can_read(obj):
            return None
        value_dict = dict()
        ctypes_dict = getattr(obj._struct.contents, self.name)
        if ctypes_dict:
            key_list = self.get_drmaa2_library().drmaa2_dict_list(ctypes_dict)
            if key_list:
                count = self.get_drmaa2_library().drmaa2_list_size(key_list)
                for i in range(count):
                    void_ptr = self.get_drmaa2_library().drmaa2_list_get(key_list, i)
                    key = cast(void_ptr, drmaa2_string).value
                    value = self.get_drmaa2_library().drmaa2_dict_get(ctypes_dict, key)
                    key = ByteString(key).decode()
                    value = ByteString(value).decode()
                    self.logger.debug('{}[{}] = {}'.format(self.name, key, value))
                    value_dict[key] = value
                self.logger.debug('Clearing key list for dict {}'.format(self.name))
                self.get_drmaa2_library().drmaa2_list_free(cast(key_list, drmaa2_string_list))
        return value_dict

    def __set__(self, obj, value_dict):
        if not self.can_write(obj):
            return
        if value_dict is None:
            value_dict = {}
        ctypes_dict = getattr(obj._struct.contents, self.name)
        if ctypes_dict:
            key_list = self.get_drmaa2_library().drmaa2_dict_list(ctypes_dict)
            if key_list:
                count = self.get_drmaa2_library().drmaa2_list_size(key_list)
                self.logger.debug('Clearing dict {} (size {})'.format(self.name, count))
                for i in range(count):
                    void_ptr = self.get_drmaa2_library().drmaa2_list_get(key_list, i)
                    key = cast(void_ptr, drmaa2_string).value
                    ExceptionMapper.check_status_code(self.get_drmaa2_library().drmaa2_dict_del(ctypes_dict, key))
        else:
            self.logger.debug('Creating dict {}'.format(self.name))
            ctypes_dict = self.get_drmaa2_library().drmaa2_dict_create(drmaa2_dict_entryfree())
            setattr(obj._struct.contents, self.name, ctypes_dict)

        value_dict = {ByteString(k).encode(): ByteString(v).encode() for (k, v) in value_dict.items()}
        for (k, v) in value_dict.items():
            self.logger.debug('{}[{}] = {}'.format(self.name, k, v))
            ExceptionMapper.check_status_code(self.get_drmaa2_library().drmaa2_dict_set(ctypes_dict, k, v))
        # this assures proper memory management in python 3
        obj._dict[self.name] = value_dict


class Drmaa2ImplSpecDescriptor(Drmaa2Descriptor):
    """ A descriptor for implementation specific fields. """

    logger = LogManager.get_instance().get_logger('Drmaa2ImplSpecDescriptor')

    def __init__(self, name='implementationSpecific'):
        Drmaa2Descriptor.__init__(self, name)

    def __get__(self, obj, type=None):
        if not self.can_read(obj):
            return None
        implSpecDict = {}
        for key in obj.get_implementation_specific_keys():
            try:
                value = obj.get_impl_spec_key_value(key)
            except UnsupportedAttribute as ex:
                # Okay, attribute was not set.
                value = None
            if value is not None:
                implSpecDict[key] = value
        return implSpecDict

    def __set__(self, obj, implSpecDict):
        if not self.can_write(obj):
            return
        for key in obj.get_implementation_specific_keys():
            value = implSpecDict.get(key)
            if value is not None:
                obj.set_impl_spec_key_value(key, value)


#######################################################################
# Test.
if __name__ == '__main__':
    o = Drmaa2BoolDescriptor('boolField')



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import random
from .utils import generate_random_string
from .utils import needs_uge
from drmaa2 import JobSession
from drmaa2 import JobInfo


@needs_uge
def test_list_session_names():
    session_names = JobSession.list_session_names()
    assert (type(session_names) == type([]))
    print('\nThere are %s existing sessions: %s' % (len(session_names), session_names))


def test_new_session():
    session_name = generate_random_string()
    existing_session_names = JobSession.list_session_names()
    js = JobSession(session_name)
    session_names = JobSession.list_session_names()
    assert (len(session_names) == len(existing_session_names) + 1)
    print('\nCreated new session: %s' % (session_name))


def test_existing_session():
    session_name = generate_random_string()
    existing_session_names = JobSession.list_session_names()
    js = JobSession(session_name, destroy_on_exit=False)
    session_names = JobSession.list_session_names()
    assert (len(session_names) == len(existing_session_names) + 1)
    print('\nCreated new session: %s' % (session_name))
    del js
    js = JobSession(session_name)
    session_names2 = JobSession.list_session_names()
    assert (len(session_names) == len(session_names2))
    print('Opened existing session: %s' % (session_name))
    del js
    session_names3 = JobSession.list_session_names()
    assert (len(session_names3) == len(existing_session_names))
    print('Closed session: %s' % (session_name))


def test_destroy_session():
    session_names = JobSession.list_session_names()
    print('\nExisting session names: %s' % session_names)
    for name in session_names:
        print('Destroying session: %s' % name)
        JobSession.destroy_by_name(name)
    session_names = JobSession.list_session_names()
    print('Remaining session names: %s' % session_names)
    assert (len(session_names) == 0)


def test_run_job():
    session_name = generate_random_string()
    js = JobSession(session_name)
    j = js.run_job({'remote_command': '/bin/sleep', 'args': ['5']})
    j_id = j.id
    assert (len(j_id) > 0)
    print('\nSubmitted job id: %s' % j_id)


def test_run_bulk_jobs():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5']}, 1, 10, 5, 2)
    jl = ja.job_list
    ja_id = ja.id
    assert (len(ja_id) > 0)
    assert (len(jl) == 2)
    print('\nSubmitted job array id: %s' % ja_id)


def test_wait_any_started():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5']}, 1, 10, 5, 2)
    jl = ja.job_list
    jl_ids = list(map(lambda j: j.id, jl))
    print('\nWaiting on start of any job with id from %s' % jl_ids)
    j = js.wait_any_started(jl)
    assert (j.id in jl_ids)
    print('Job %s started' % j)


def test_wait_any_terminated():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5']}, 1, 10, 5, 2)
    jl = ja.job_list
    jl_ids = list(map(lambda j: j.id, jl))
    print('\nWaiting on termination of any job with id from %s' % jl_ids)
    j = js.wait_any_terminated(jl)
    assert (j.id in jl_ids)
    print('Job %s terminated' % j)


def test_wait_all_started():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5']}, 1, 10, 5, 2)
    jl = ja.job_list
    jl_ids = list(map(lambda j: j.id, jl))
    jl_ids.sort()
    print('\nWaiting on start of all jobs: %s' % jl_ids)
    jl2 = js.wait_all_started(jl)
    jl2_ids = list(map(lambda j: j.id, jl2))
    jl2_ids.sort()
    assert (jl_ids == jl2_ids)
    print('Jobs started: ' % jl2_ids)


def test_wait_all_terminated():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5']}, 1, 10, 5, 2)
    jl = ja.job_list
    jl_ids = list(map(lambda j: j.id, jl))
    jl_ids.sort()
    print('\nWaiting on termination of all jobs: %s' % jl_ids)
    jl2 = js.wait_all_terminated(jl)
    jl2_ids = list(map(lambda j: j.id, jl2))
    jl2_ids.sort()
    assert (jl_ids == jl2_ids)
    print('Jobs terminated: ' % jl2_ids)


def test_get_job_array():
    session_name = generate_random_string()
    js = JobSession(session_name)
    ja = js.run_bulk_jobs({'remote_command': '/bin/sleep', 'args': ['5']}, 1, 10, 5, 2)
    ja2 = js.get_job_array(ja.id)
    assert (ja.id == ja2.id)
    print('\nRetrieved job array: %s' % ja2)


def test_get_job_categories():
    session_name = generate_random_string()
    js = JobSession(session_name)
    job_categories = js.get_job_categories()
    assert (type(job_categories) == type([]))
    print('\nThere are %s job categories: %s' % (len(job_categories), job_categories))


def test_get_jobs():
    js = JobSession('js-01')
    j_name = 'drmaa2python-%s' % int(random.uniform(0, 1000))
    j = js.run_job({'remote_command': '/bin/sleep', 'args': ['10'], 'job_name': j_name})
    print('\nSubmitted job: %s' % j)
    ji = j.get_info()
    print('Retrieving jobs matching job info %s' % ji)
    ji2 = JobInfo({'job_id': ji.job_id})
    j_list = js.get_jobs(ji2)
    print('Got jobs: %s' % j_list)
    assert (len(j_list) >= 1)



# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__
from __future__ import with_statement, print_function
import sys

# Work around http://bugs.python.org/issue15881#msg170215
try:
    import multiprocessing
except ImportError:
    pass

try:
    from setuptools import setup

    extra = dict(include_package_data=True)
except ImportError:
    from distutils.core import setup

    extra = {}

import os

from drmaa2 import __version__

if sys.version_info <= (2, 7):
    error = "ERROR: UGE DRMAA2 API requires Python Version 2.7 or above...exiting."
    print(error, file=sys.stderr)
    sys.exit(1)

with open("README.rst") as f:
    long_description = f.read()

setup(name='uge-drmaa2',
      version=__version__,
      description='UGE DRMAA2 Python API',
      long_description=long_description,
      # long_description_content_type='text/markdown',
      author='Univa',
      author_email='info@univa.com',
      test_suite='test',
      url='https://www.univa.com',
      packages=['drmaa2'],
      install_requires=[],
      package_data={
      },
      license='Apache 2.0',
      platforms='Posix; MacOS X',
      classifiers=[
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
      ],
      **extra
      )



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import os
import logging
from .singleton import Singleton


class LogManager(Singleton):
    """ 
    Singleton class for managing library logging output. Log level 
    can be set either programatically, or via the DRMAA2_LOG_LEVEL 
    environment variable. Valid levels are the same as for the python 
    logging module: DEBUG, INFO, WARNING, ERROR, CRITICAL.
    """

    LOG_LEVEL_ENV_VAR = 'DRMAA2_LOG_LEVEL'
    LOG_LEVEL_MAP = {
        'CRITICAL': logging.CRITICAL,
        'ERROR': logging.ERROR,
        'WARNING': logging.WARNING,
        'INFO': logging.INFO,
        'DEBUG': logging.DEBUG,
        'NOTSET': logging.NOTSET,
    }
    LOG_MESSAGE_FORMAT = '%(asctime)s %(levelname)s %(filename)s:%(lineno)d:  %(message)s'

    __instance = None

    def __init__(self):
        """
        Constructor. 

        >>> lm = LogManager()
        """
        if LogManager.__instance:
            return
        LogManager.__instance = self
        self.configure()

    def configure(self, level=None):
        """
        Confgure the log level. 

        :param level: Log level; if not provided, the DRMAA2_LOG_LEVEL environment variable will be used to set the level.
        :type level: str

        >>> LogManager.get_instance.configure('DEBUG')
        """
        level = level or os.environ.get(self.LOG_LEVEL_ENV_VAR)
        if level:
            log_level = self.LOG_LEVEL_MAP.get(level.upper(), logging.NOTSET)
            logging.basicConfig(level=log_level, format=self.LOG_MESSAGE_FORMAT)
        else:
            logging.basicConfig(format=self.LOG_MESSAGE_FORMAT)

    @classmethod
    def get_logger(cls, name=None):
        """ 
        Get logger with a given name.

        :param name: Logger name; if not provided, class name will be used.
        :type name: str

        :returns: Logger object.

        >>> logger = LogManager.get_logger('MyClass')
        """
        name = name or cls.__name__
        return logging.getLogger(name)



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from .utils import generate_random_string
from .utils import generate_random_int
from drmaa2 import Sudo
from drmaa2 import Drmaa2Exception


def test_username_attr():
    username = generate_random_string()
    s = Sudo()
    s.username = username
    assert (s.username == username)
    s2 = Sudo(username=username)
    assert (s2.username == username)
    s3 = Sudo(sudo_dict={'username': username})
    assert (s3.username == username)
    print('\nSudo object with username: %s' % (username))


def test_groupname_attr():
    groupname = generate_random_string()
    s = Sudo()
    s.groupname = groupname
    assert (s.groupname == groupname)
    s2 = Sudo(groupname=groupname)
    assert (s2.groupname == groupname)
    s3 = Sudo(sudo_dict={'groupname': groupname})
    assert (s3.groupname == groupname)
    print('\nSudo object with groupname: %s' % (groupname))


def test_uid_attr():
    uid = generate_random_int()
    s = Sudo()
    s.uid = uid
    assert (s.uid == uid)
    s2 = Sudo(uid=uid)
    assert (s2.uid == uid)
    s3 = Sudo(sudo_dict={'uid': uid})
    assert (s3.uid == uid)
    print('\nSudo object with uid: %s' % (uid))


def test_gid_attr():
    gid = generate_random_int()
    s = Sudo()
    s.gid = gid
    assert (s.gid == gid)
    s2 = Sudo(gid=gid)
    assert (s2.gid == gid)
    s3 = Sudo(sudo_dict={'gid': gid})
    assert (s3.gid == gid)
    print('\nSudo object with gid: %s' % (gid))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__


import copy
import re
from ctypes import cast
from ctypes import c_void_p
from ctypes import pointer

from .drmaa2_constants import PY_STRING_TYPE
from .drmaa2_constants import PY_BYTES_TYPE
from .drmaa2_constants import ListType
from .byte_string import ByteString
from .drmaa2_ctypes import drmaa2_string
from .drmaa2_ctypes import drmaa2_list_entryfree

from .log_manager import LogManager
from .library_manager import LibraryManager
from .exception_mapper import ExceptionMapper

from .drmaa2_object_descriptors import Drmaa2BoolDescriptor
from .drmaa2_object_descriptors import Drmaa2EnumDescriptor
from .drmaa2_object_descriptors import Drmaa2TimeDescriptor
from .drmaa2_object_descriptors import Drmaa2IntDescriptor
from .drmaa2_object_descriptors import Drmaa2LongDescriptor
from .drmaa2_object_descriptors import Drmaa2LongLongDescriptor
from .drmaa2_object_descriptors import Drmaa2FloatDescriptor
from .drmaa2_object_descriptors import Drmaa2CharBufferDescriptor
from .drmaa2_object_descriptors import Drmaa2StringDescriptor
from .drmaa2_object_descriptors import Drmaa2StringListDescriptor
from .drmaa2_object_descriptors import Drmaa2DictDescriptor
from .drmaa2_object_descriptors import Drmaa2ImplSpecDescriptor


class Drmaa2Object(object):
    """ Base class for all high-level objects. """

    BoolDescriptor = Drmaa2BoolDescriptor
    EnumDescriptor = Drmaa2EnumDescriptor
    TimeDescriptor = Drmaa2TimeDescriptor
    CharBufferDescriptor = Drmaa2CharBufferDescriptor
    StringDescriptor = Drmaa2StringDescriptor
    StringListDescriptor = Drmaa2StringListDescriptor
    DictDescriptor = Drmaa2DictDescriptor
    IntDescriptor = Drmaa2IntDescriptor
    LongDescriptor = Drmaa2LongDescriptor
    LongLongDescriptor = Drmaa2LongLongDescriptor
    FloatDescriptor = Drmaa2FloatDescriptor
    ImplSpecDescriptor = Drmaa2ImplSpecDescriptor

    logger = LogManager.get_instance().get_logger('Drmaa2Object')
    drmaa2_lib = None
    implementation_specific_keys = None
    implementation_specific_attrs = None
    attribute_names = None

    def __init__(self, struct=None):
        """ Constructor. """
        self._struct = struct
        self._dict = {}
        self._read_only = False

    def __eq__(self, other):
        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        return not self.__eq__(other)

    @classmethod
    def get_drmaa2_library(cls):
        if not cls.drmaa2_lib:
            cls.drmaa2_lib = LibraryManager.get_instance().get_drmaa2_library()
        return cls.drmaa2_lib

    @classmethod
    def scrub_dict(cls, d):
        """ Remove empty keys from the dictionary. """
        return {k: v for (k, v) in d.items() if v is not None}

    def get_impl_spec_key_value(self, key):
        """ Get value for an implementation specific key. """
        ctypes_string = self.drmaa2_lib.drmaa2_get_instance_value(cast(self._struct, c_void_p), key.encode())
        # ExceptionMapper.check_last_error_code()
        return self.to_py_string(ctypes_string)

    def set_impl_spec_key_value(self, key, value):
        """ Get value for an implementation specific key. """
        ExceptionMapper.check_status_code(
            self.drmaa2_lib.drmaa2_set_instance_value(cast(self._struct, c_void_p), key.encode(), value.encode()))

    def init_impl_spec_key_values(self):
        """ Initialize values for implementation specific keys. """
        for key in self.get_implementation_specific_keys():
            self.set_impl_spec_key_value(key, '')

    @classmethod
    def create_from_ctypes(cls, ctypes_struct):
        """ Convert ctypes struct to DRMAA2 object. """
        if ctypes_struct:
            obj = cls()
            obj._struct = ctypes_struct
            return obj
        else:
            return None

    @classmethod
    def create_from_dict(cls, d):
        """ 
        Create from dictionary. 
       
        :param d: Input dictionary.
        :type d: dict
        """
        if isinstance(d, cls):
            return d

        obj = cls()
        obj.from_dict(d)
        return obj

    def from_dict(self, d):
        """ 
        Set from dictionary. 
       
        :param d: Input dictionary.
        :type d: dict
        """
        if not d:
            return

        if not self._struct:
            self._dict.join(d)
            return

        for (key, value) in d.items():
            self.logger.debug('Setting {}={}'.format(key, str(value)))
            setattr(self, key, value)

    def to_dict(self):
        """ 
        Conversion to dictionary. 

        :returns: Dictionary representing object's structure.

        >>> v = Version.get_drms_version()
        >>> print(v.to_dict())
        {'implementation_specific': {}, 'major': '8', 'minor': '6.0'}
        """
        d = dict()
        if not self._struct:
            for (k, v) in self._dict.items():
                if v is not None:
                    d[k] = v
                    if type(v) == PY_STRING_TYPE:
                        pass
                    elif type(v) == PY_BYTES_TYPE:
                        try:
                            d[k] = v.decode()
                        except AttributeError:
                            pass
        else:
            attributes = self.get_attribute_names(self._struct)
            for attr_name in attributes:
                v = getattr(self, attr_name)
                if v is not None:
                    d[attr_name] = v
        return d

    @classmethod
    def get_attribute_names(cls, struct):
        if cls.attribute_names is None and struct is not None:
            attributes = [field[0] for field in struct.contents._fields_]
            cls.attribute_names = []
            for attr_name in sorted(attributes):
                # Convert aXY to a_XY
                attr_name2 = re.sub('([a-z])([A-Z]{2,})', r'\1_\2', attr_name)
                # Convert Xa to _Xa
                attr_name2 = re.sub('([A-Z][a-z]{1,})', r'_\1', attr_name2).lower()
                cls.attribute_names.append(attr_name2)
        return cls.attribute_names

    def __str__(self):
        """ Conversion to string. """
        return str(self.to_dict())

    def __repr__(self):
        """ Object representation. """
        return '%s(%s)' % (self.__class__.__name__, self.__str__())

    @classmethod
    def to_py_dict(cls, ctypes_dict):
        py_dict = dict()
        if ctypes_dict:
            drmaa2_lib = cls.get_drmaa2_library()
            ctypes_list = drmaa2_lib.drmaa2_dict_list(ctypes_dict)
            list_size = drmaa2_lib.drmaa2_list_size(ctypes_list)
            if list_size < 0:
                ExceptionMapper.check_last_error_code()
            for i in range(list_size):
                void_ptr = drmaa2_lib.drmaa2_list_get(ctypes_list, i)
                key_ptr = cast(void_ptr, drmaa2_string).value
                value_ptr = drmaa2_lib.drmaa2_dict_get(ctypes_dict, key_ptr)
                key = ByteString(key_ptr).decode()
                value = ByteString(value_ptr).decode()
                py_dict[key] = value
            drmaa2_lib.drmaa2_list_free(pointer(c_void_p(ctypes_list)))
            drmaa2_lib.drmaa2_dict_free(pointer(c_void_p(ctypes_dict)))
        return py_dict

    @classmethod
    def to_py_string_list(cls, ctypes_list):
        py_list = list()
        if ctypes_list:
            drmaa2_lib = cls.get_drmaa2_library()
            list_size = drmaa2_lib.drmaa2_list_size(ctypes_list)
            if list_size < 0:
                ExceptionMapper.check_last_error_code()
            for i in range(list_size):
                void_ptr = drmaa2_lib.drmaa2_list_get(ctypes_list, i)
                value = ByteString(cast(void_ptr, drmaa2_string).value).decode()
                py_list.append(value)
            drmaa2_lib.drmaa2_list_free(pointer(c_void_p(ctypes_list)))
        return py_list

    @classmethod
    def to_py_string(cls, ctypes_string, free_original=False):
        py_string = ByteString(ctypes_string.value).decode()
        if free_original:
            cls.get_drmaa2_library().drmaa2_string_free(pointer(ctypes_string))
        if not py_string:
            py_string = None
        return py_string

    @classmethod
    def to_ctypes_string_list(cls, py_list):
        drmaa2_lib = cls.get_drmaa2_library()
        ctypes_list = drmaa2_lib.drmaa2_list_create(int(ListType.STRINGLIST), drmaa2_list_entryfree())
        value_list = [ByteString(v).encode() for v in py_list]
        for i in range(len(value_list)):
            v = value_list[i]
            ExceptionMapper.check_status_code(drmaa2_lib.drmaa2_list_add(ctypes_list, v))
        return ctypes_list

    @classmethod
    def get_implementation_specific_keys(cls):
        return cls.implementation_specific_keys


#######################################################################
# Test.
if __name__ == '__main__':
    o = Drmaa2Object()



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import datetime
from .utils import generate_random_string
from .utils import generate_random_int
from drmaa2 import ReservationSession
from drmaa2 import ReservationInfo
from drmaa2.drmaa2_constants import POSIX_EPOCH


def test_reservation_info_from_reservation():
    rs = ReservationSession('rs-01')
    reservation_name = 'r.%s' % generate_random_string()
    r = rs.request_reservation({'reservation_name': reservation_name, 'duration': 100})
    ri = r.get_info()
    assert (ri.reservation_name == reservation_name)
    print('\nReservation info from reservation: %s' % (ri))


def test_get_implementation_specific_keys():
    keys = ReservationInfo.get_implementation_specific_keys()
    assert (len(keys) > 0)
    print('\nReservation info implementation specific keys: %s' % (keys))


def test_reservation_id_attr():
    ri = ReservationInfo()
    reservation_id = generate_random_string()
    ri.reservation_id = reservation_id
    assert (ri.reservation_id == reservation_id)
    print('\nReservation info with reservation_id: %s' % (reservation_id))


def test_reservation_name_attr():
    ri = ReservationInfo()
    reservation_name = generate_random_string()
    ri.reservation_name = reservation_name
    assert (ri.reservation_name == reservation_name)
    print('\nReservation info with reservation_name: %s' % (reservation_name))


def test_reserved_start_time_attr():
    reserved_start_time = datetime.datetime.now()
    ri = ReservationInfo()
    ri.reserved_start_time = reserved_start_time
    assert (int((ri.reserved_start_time - POSIX_EPOCH).total_seconds()) == int(
        (reserved_start_time - POSIX_EPOCH).total_seconds()))
    print('\nReservation info object with reserved_start_time: %s' % (reserved_start_time))


def test_reserved_end_time_attr():
    reserved_end_time = datetime.datetime.now() + datetime.timedelta(hours=24)
    ri = ReservationInfo()
    ri.reserved_end_time = reserved_end_time
    assert (int((ri.reserved_end_time - POSIX_EPOCH).total_seconds()) == int(
        (reserved_end_time - POSIX_EPOCH).total_seconds()))
    print('\nReservation info object with reserved_end_time: %s' % (reserved_end_time))


def test_users_acl_attr():
    users_acl = []
    n_users_acl = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_users_acl):
        users_acl.append(generate_random_string())
    ri = ReservationInfo()
    ri.users_acl = users_acl
    assert (ri.users_acl == users_acl)
    print('\nreservation info object with %s users_acl: %s' % (len(users_acl), users_acl))


def test_reserved_slots_attr():
    ri = ReservationInfo()
    reserved_slots = generate_random_int(lower_bound=1, upper_bound=1024)
    ri.reserved_slots = reserved_slots
    assert (ri.reserved_slots == reserved_slots)
    print('\nReservation info with reserved_slots: %s' % (reserved_slots))


def test_reserved_machines_attr():
    reserved_machines = []
    n_reserved_machines = generate_random_int(lower_bound=1, upper_bound=5)
    for i in range(0, n_reserved_machines):
        reserved_machines.append(generate_random_string())
    ri = ReservationInfo()
    ri.reserved_machines = reserved_machines
    assert (ri.reserved_machines == reserved_machines)
    print('\nreservation info object with %s reserved_machines: %s' % (len(reserved_machines), reserved_machines))


def test_implementation_specific_attr():
    implementation_specific = {}
    keys = ReservationInfo.get_implementation_specific_keys()
    for k in keys:
        v = generate_random_string()
        implementation_specific[k] = v
    ri = ReservationInfo()
    ri.implementation_specific = implementation_specific
    assert (ri.implementation_specific == implementation_specific)
    print('\nReservation info object with implementation_specific: %s' % (implementation_specific))
